<?php

namespace Modules\ReportsModule\Http\Controllers;

use stdClass;
use App\User;
use App\Category;
use App\Product;
use Exception;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\PaginatedResourceResponse;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Connector\Http\Controllers\Api\ApiController;
use Modules\ReportsModule\Transformers\CollectionResource;
use Illuminate\Support\Str;

class FinanceReportApiController extends ApiController
{
    /**
     * Display a listing of the resource.
     * @return JsonResponse
     * @throws Exception
     */
    public function index()
    {
    // Sending locations according to login user
    $user = Auth::user();
    $business_id = $user->business_id;
    $permitted_locations = $user->permitted_locations($business_id);
    $filters = request()->only([
            'location_id', 'location_type', 'ratio_filter', 'start_date' , 'end_date' ,
            'not_for_selling', 'is_inactive', 'search', 'managed_stock', 'per_page','page', 'category_id' , 'sub_category_id'
        ]);

    $transactions = DB::table('transactions as t')
                    ->select(
                        DB::raw(' t.id as transaction_id '),
                        DB::raw(' t.transaction_date as transaction_date '),
                        DB::raw(' t.location_id as transaction_location_id '),
                        DB::raw(' t.invoice_no as transaction_invoice_no '),
                        DB::raw(' t.payment_status as transaction_payment_status '),
                        DB::raw(' t.customer_group_id as transaction_customer_group_id '),
                        DB::raw(' t.contact_id as transaction_contact_id '),
                        DB::raw(' t.total_before_tax as transaction_total_before_tax '),
                        DB::raw(' t.tax_id as transaction_tax_id '),
                        DB::raw(' t.tax_amount as transaction_tax_amount '),
                        DB::raw('IF(t.discount_type = "percentage", (t.discount_amount/100)*t.total_before_tax , t.discount_amount) as transaction_discount_amount'),
                        DB::raw(' t.final_total as transaction_final_total ')
                        )
                    ->leftjoin('business_locations as bl','bl.id', '=', 't.location_id')
                    ->where('t.type', '=' , 'sell')
                    ->where('t.status', '=' , 'final');


                    // Filters
                if (!empty($filters['location_id']))
                    $transactions->whereIn('t.location_id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $transactions->whereIn('t.location_id', $permitted_locations);

                if (!empty($filters['location_type']))
                {
                    if($filters['location_type'] == 'null') {
                        $transactions->whereNull('bl.custom_field1');
                    } else {
                        $transactions->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                    }
                }

                if (!empty($filters['start_date']))
                    $transactions->whereDate('t.transaction_date', '>=', $filters['start_date']);
                if (!empty($filters['end_date']))
                    $transactions->whereDate('t.transaction_date', '<=', $filters['end_date']);

                $transactions = $transactions->get();
                
    // Defines
    $transactions_details = [];
    $transactions_data = [];
    $sell_lines = [];
    $obj = new stdClass;
    $sell_line_unit_price_before_discount = 0;
    // End Defines
    
        // return($transactions);
    foreach ($transactions as $transaction)
    {

        $transaction_sell_lines = DB::table('transaction_sell_lines as tsl')
                    ->select(
                        DB::raw(' tsl.product_id as product_id '),
                        DB::raw(' p.category_id as product_category_id '),
                        DB::raw(' p.sub_category_id as product_sub_category_id '),
                        DB::raw(' p.enable_stock AS product_enable_stock'),
                        DB::raw(' (tsl.quantity - tsl.quantity_returned) as sell_line_sold_qty '),
                        DB::raw('IF(tsl.line_discount_type = "percentage", (tsl.line_discount_amount/100)*tsl.unit_price_before_discount , tsl.line_discount_amount) as sell_line_discount_amount'),
                        DB::raw(' tsl.tax_id as sell_line_tax_id '),
                        DB::raw(' tsl.item_tax as sell_line_item_tax '),
                        DB::raw(' tsl.unit_price_before_discount as sell_line_unit_price_before_discount '),
                        DB::raw(' v.dpp_inc_tax as product_purchase_price_inc_tax'),
                        DB::raw(' tsl_pl.purchase_line_id as tsl_pl_purchase_line_id '),
                        DB::raw(' (tsl_pl.quantity - tsl_pl.qty_returned) as tsl_pl_sold_qty ')
                        )
                    ->leftjoin('products as p','p.id', '=', 'tsl.product_id')
                    ->leftjoin('variations as v','v.id', '=', 'tsl.variation_id')
                    ->leftjoin('transaction_sell_lines_purchase_lines as tsl_pl','tsl.id', '=', 'tsl_pl.sell_line_id')
                    ->where('tsl.transaction_id', $transaction->transaction_id)
                    ->get();
                    
            // return($transaction_sell_lines);
        foreach($transaction_sell_lines as $transaction_sell_line)
        {
            if($transaction_sell_line->product_enable_stock == 0)
            {   
                $product_id = $transaction_sell_line->product_id;
                $product_category_id = $transaction_sell_line->product_category_id;
                $product_sub_category_id = $transaction_sell_line->product_sub_category_id;
                $sell_line_sold_qty = $transaction_sell_line->sell_line_sold_qty;
                $sell_line_discount_amount = $transaction_sell_line->sell_line_discount_amount;
                $sell_line_tax_id = $transaction_sell_line->sell_line_tax_id;
                $sell_line_item_tax = $transaction_sell_line->sell_line_item_tax;
                $product_cost = $transaction->product_purchase_price_inc_tax;
                $sell_line_unit_price_before_discount = $transaction_sell_lines->sell_line_unit_price_before_discount;
            }
            else
            {
                $purchase_lines = DB::table('purchase_lines as pl')
                        ->select(
                            DB::raw(' pl.purchase_price_inc_tax AS pl_product_cost ')
                            )
                        ->where('pl.id', $transaction_sell_line->tsl_pl_purchase_line_id)
                        ->get();
                        
                if($transaction_sell_line->tsl_pl_purchase_line_id != 0 and $purchase_lines->count() > 0 )
                {
                    
                    $product_id = $transaction_sell_line->product_id;
                    $product_category_id = $transaction_sell_line->product_category_id;
                    $product_sub_category_id = $transaction_sell_line->product_sub_category_id;
                    $sell_line_sold_qty = $transaction_sell_line->sell_line_sold_qty;
                    $sell_line_discount_amount = $transaction_sell_line->sell_line_discount_amount;
                    $sell_line_tax_id = $transaction_sell_line->sell_line_tax_id;
                    $sell_line_item_tax = $transaction_sell_line->sell_line_item_tax;
                    $product_cost = $purchase_lines[0]->pl_product_cost;
                    $sell_line_unit_price_before_discount = $transaction_sell_lines->sell_line_unit_price_before_discount;
                
                    $sold_qty = $transaction_sell_line->tsl_pl_sold_qty;
                }
                else
                {
                    if($transaction_sell_line->tsl_pl_sold_qty == null)
                    {
                        $sold_qty = $transaction->tsl_sold_qty;
                    }
                    else
                    {
                        $sold_qty = $transaction->tsl_pl_sold_qty;
                    }
                    
                    $product_id = $transaction_sell_line->product_id;
                    $product_category_id = $transaction_sell_line->product_category_id;
                    $product_sub_category_id = $transaction_sell_line->product_sub_category_id;
                    $sell_line_sold_qty = $transaction_sell_line->sell_line_sold_qty;
                    $sell_line_discount_amount = $transaction_sell_line->sell_line_discount_amount;
                    $sell_line_tax_id = $transaction_sell_line->sell_line_tax_id;
                    $sell_line_item_tax = $transaction_sell_line->sell_line_item_tax;
                    $product_cost = $transaction_sell_lines->product_purchase_price_inc_tax;
                    $sell_line_unit_price_before_discount = $transaction_sell_lines->sell_line_unit_price_before_discount;
                }
            }
            
        }
            
            
        array_push($transactions_details, (object)[
            'product_id' => $product_id,
            'product_category_id' => $product_category_id,
            'product_sub_category_id' => $product_sub_category_id,
            'sell_line_sold_qty' => $sell_line_sold_qty,
            'sell_line_discount_amount' => $sell_line_discount_amount,
            'sell_line_tax_id' => $sell_line_tax_id,
            'sell_line_item_tax' => $sell_line_item_tax,
            'product_cost' => $product_cost,
            'sell_line_unit_price_before_discount' => $sell_line_unit_price_before_discount,

            'sold_qty' => $sold_qty,

            
        ]);
            
            
            
            
            
            
        $transaction_payment_lines = DB::table('transaction_payments as tpy')
                    ->select(
                        DB::raw(' tpy.is_return as returned '),
                        // DB::raw(' tpy.amount as amount '),
                        DB::raw('IF(tpy.is_return = 1, (tpy.amount)*(-1) , tpy.amount) as amount'),
                        DB::raw(' tpy.method as method ')
                        )
                    ->where('tpy.transaction_id', $transaction->transaction_id)
                    ->get();

        array_push($transactions_data, (object)[
            'transaction_id' => $transaction->transaction_id,
            'transaction_location_id' => $transaction->transaction_location_id,
            'transaction_date' => $transaction->transaction_date,
            'transaction_invoice_no' => $transaction->transaction_invoice_no,
            'transaction_payment_status' => $transaction->transaction_payment_status,
            'transaction_contact_id' => $transaction->transaction_contact_id,
            'transaction_customer_group_id' => $transaction->transaction_customer_group_id,
            'transaction_total_before_tax' => $transaction->transaction_total_before_tax,
            'transaction_tax_id' => $transaction->transaction_tax_id,
            'transaction_tax_amount' => $transaction->transaction_tax_amount,
            'transaction_discount_amount' => $transaction->transaction_discount_amount,
            'transaction_final_total' => $transaction->transaction_final_total,
            'sell_lines' => $transactions_details,
            'payment_lines' => $transaction_payment_lines,
            
        ]);

    }

    if(!empty($transactions_data))
    {
        $perPage = 10;
        $skip = 0;
        if(!empty($filters['per_page']))
        {
            if (($filters['per_page']) != -1)
            {
                $perPage = $filters['per_page'];
            }
            else
            {
                 $perPage = count($transactions_data);
            }
        }


        $sells = $this->paginate($transactions_data, $perPage);
        $sells->appends(Arr::except($filters, ['page']));

        $response = ((new PaginatedResourceResponse(CollectionResource::make($sells)))->toResponse(request()));
        $data = $response->getData();
        
      
        
        
        $response->setData($data);
        return $response;
    }
    else{
        return $transactions_data;
    }


    }


    /**
     * Create a new command instance.
     *
     * @return string
     */
    public function queryToSql($query)
    {
        return vsprintf(str_replace(array('?'), array('\'%s\''), $query->toSql()), $query->getBindings());
    }

    /**
     * The attributes that are mass assignable.
     *
     *
     * @param $items
     * @param int $perPage
     * @param null $page
     * @param array $options
     * @return LengthAwarePaginator
     */
    public function paginate($items, $perPage = 5, $page = null, $options = [])
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(), $perPage, $page, $options);
    }
}
