<?php
namespace Modules\ReportsModule\Http\Controllers;

use stdClass;
use App\User;
use App\Category;
use App\Product;
use Exception;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\PaginatedResourceResponse;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Connector\Http\Controllers\Api\ApiController;
use Modules\ReportsModule\Transformers\CollectionResource;
use Illuminate\Support\Str;


class PosProductsApiController extends ApiController
{
    /**
     * Display a listing of the resource.
     * @return JsonResponse
     * @throws Exception
     */

    public function index()
    {


        /** @var User $user */
        $user = Auth::user();
        $business_id = $user->business_id;
        $permitted_locations = $user->permitted_locations($business_id);
        $filters = request()->only([
            'location_id', 'location_type', 'ratio_filter',
            'not_for_selling', 'is_inactive', 'search', 'managed_stock', 'per_page','page', 'category_id' , 'sub_category_id'
        ]);
        

        $products = DB::table('variations AS v')
                ->select(
                    DB::raw('v.product_id AS product_id'),
                    DB::raw('v.id AS variation_id'),
                    DB::raw('IF(v.name = "DUMMY", p.name , CONCAT(p.name, " + ", v.name)) as product_name_ar'),
                    DB::raw('IF(p.product_custom_field1 IS NULL or p.product_custom_field1 = "", IF(v.name = "DUMMY", p.name , CONCAT(p.name, " + ", v.name)) , IF(v.name = "DUMMY", p.product_custom_field1 , CONCAT(p.product_custom_field1, " + ", v.name))) as product_name_en'),
                    DB::raw('v.name AS variation_name'),
                    DB::raw('v.sub_sku AS sku')
                )
                ->leftjoin('products as p','v.product_id', '=', 'p.id')
                ->leftjoin('purchase_lines as pl','pl.variation_id', '=', 'v.id')
                ->leftjoin('transactions as t','t.id', '=', 'pl.transaction_id')
                ->where('p.business_id', $business_id)
                ->where('p.id', "!=" , 1)
                ->where('p.is_inactive', 0)
                ->where('t.status', 'received')
                // ->where('p.not_for_selling', 0)
                ->where(\DB::raw('(pl.quantity - pl.quantity_sold - pl.quantity_adjusted + pl.quantity_returned)'), '>' , 0)
                ->groupBy('variation_id')
                ->orderBy('v.sub_sku', 'asc');



                // Filters
                if (!empty($filters['location_id']))
                    $products->whereIn('t.location_id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $products->whereIn('l.location_id', $permitted_locations);

                //  if (!empty($filters['location_type']))
                //  {
                //      if($filters['location_type'] == 'null') {
                //          $products->whereNull('bl.custom_field1');
                //      } else {
                //          $products->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                //      }
                //  }

                
                // if (!empty($filters['category_id'])) {
                //     $products->where('p.category_id', $filters['category_id']);
                // }

                // if (!empty($filters['sub_category_id'])) {
                //     $products->where('p.sub_category_id', $filters['sub_category_id']);
                // }

                // if (!empty($filters['search']))
                //     $products->where(function ($products) use ($filters) {
                //         $products
                //             ->where('p.sku', 'like', "%{$filters['search']}%")
                //             ->orWhere('p.name', 'like', "%{$filters['search']}%")
                //             ->orWhere('p.product_custom_field1', 'like', "%{$filters['search']}%")
                //             ->orWhere('v.name', 'like', "%{$filters['search']}%")
                //             ->orWhere('v.sub_sku', 'like', "%{$filters['search']}%");
                //     });
                
                // End Filters

// return($products->toSql());
                $products = $products->get();
// return($products);
                
                $response = [];
                foreach($products as $product)
                {
                    $products_details = DB::table('purchase_lines as pl')
                        ->select(
                            DB::raw('(pl.quantity - pl.quantity_sold - pl.quantity_adjusted + pl.quantity_returned) AS available_qty'),
                            DB::raw('pl.lot_number AS lot_number'),
                            DB::raw('pl.purchase_price_inc_tax AS v_dpp_inc_tax')
                        )
                        ->where(\DB::raw('(pl.quantity - pl.quantity_sold - pl.quantity_adjusted + pl.quantity_returned)'), '>' , 0)
                        ->where('pl.variation_id', $product->variation_id)
                        // ->where('pl.variation_id', 4229)
                        ->get();

                    array_push($response, (object)[
                        'product_id' => $product->product_id,
                        'variation_id' => $product->variation_id,
                        'product_name_ar' => $product->product_name_ar,
                        'product_name_en' => $product->product_name_en,
                        'variation_name' => $product->variation_name,
                        'sku' => $product->sku,
                        'lot_details' => $products_details,
                    ]);
                    
                }
                

                // $response = [];
                // foreach($products as $product)
                // {
                //     if(!in_array( $product->variation_id, array_column($response, 'variation_id')) )
                //     {
                //         $lot_number = [];
                //         array_push($lot_number, (object)[
                //             'lot_number' => $product->product_id,
                //             'variation_id' => $product->variation_id,
                //             'product_name_ar' => $product->product_name_ar,
                //             'product_name_en' => $product->product_name_en,
                //             'variation_name' => $product->variation_name,
                //             'sku' => $product->sku,
                            
                //             'transaction_ref_no' => $result2->transaction_ref_no,
                //             'account_name' => $result->account_name,
                //             'user_first_name' => $result2->user_first_name,
                //             'user_last_name' => $result2->user_last_name,
                //             'contact_name' => $result2->contact_name,
                            
                //             'type' => 'a',
                //         ]);
                //         array_push($transactions_payments, (object)[
                //             'product_id' => $product->product_id,
                //             'variation_id' => $product->variation_id,
                //             'product_name_ar' => $product->product_name_ar,
                //             'product_name_en' => $product->product_name_en,
                //             'variation_name' => $product->variation_name,
                //             'sku' => $product->sku,
                            
                //             'transaction_ref_no' => $result2->transaction_ref_no,
                //             'account_name' => $result->account_name,
                //             'user_first_name' => $result2->user_first_name,
                //             'user_last_name' => $result2->user_last_name,
                //             'contact_name' => $result2->contact_name,
                            
                //             'type' => 'a',
                //         ]);
                //     }
                // }
                
                return($response);
    }

}
