<?php

namespace Modules\ReportsModule\Http\Controllers;

use stdClass;
use App\User;
use App\Category;
use App\Product;
use Exception;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\PaginatedResourceResponse;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Connector\Http\Controllers\Api\ApiController;
use Modules\ReportsModule\Transformers\CollectionResource;
use Illuminate\Support\Str;

class ProductCategoriesDataReportApiController extends ApiController
{
    /**
     * Display a listing of the resource.
     * @return JsonResponse
     * @throws Exception
     */

    public function index()
    {
        /** @var User $user */
        $user = Auth::user();
        $business_id = $user->business_id;
        $permitted_locations = $user->permitted_locations($business_id);
        $filters = request()->only([
            'location_id', 'location_type', 'ratio_filter',
            'not_for_selling', 'is_inactive', 'search', 'managed_stock', 'per_page','page',
        ]);


            $categories = DB::table('products as p')
                ->select(
                    DB::raw('c.id AS id'),
                    DB::raw('IF(c.name IS NULL, "No Category / بدون تصنيف" ,c.name) as name'),
                    // DB::raw('c.name AS name'),
                    DB::raw('c.parent_id AS parent_id')
                )
                ->leftjoin('categories as c','p.category_id', '=', 'c.id')
                ->leftjoin('product_locations as l','l.product_id', '=', 'p.id')
                ->leftjoin('business_locations as bl','bl.id', '=', 'l.location_id')
                ->where('p.business_id', $business_id);



                // Filters
                if (!empty($filters['location_id']))
                    $categories->whereIn('l.location_id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $categories->whereIn('l.location_id', $permitted_locations);

                 if (!empty($filters['location_type']))
                 {
                     if($filters['location_type'] == 'null') {
                         $categories->whereNull('bl.custom_field1');
                     } else {
                         $categories->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                     }
                 }

                if (isset($filters['not_for_selling']) && in_array($filters['not_for_selling'], [0, 1]))
                    $categories->where('p.not_for_selling', $filters['not_for_selling']);
                if (isset($filters['is_inactive']) && in_array($filters['is_inactive'], [0, 1]))
                    $categories->where('p.is_inactive', $filters['is_inactive']);
                if (isset($filters['managed_stock']) && in_array($filters['managed_stock'], [0, 1]))
                    $categories->where('p.enable_stock', $filters['managed_stock']);

                // End Filters

                $categories = $categories->get();
                
                $groups = $categories->groupBy('id');
                // return($groups);
                $categories = $groups->map(function ($row) {
                    return [
                      'id' => $row->first()->id,
                      'name' => $row->first()->name,
                      'parent_id' => $row->first()->parent_id,
                    ];
        
                });
                $categories = $categories->values();
                

        // Defines
        $products_data = [];
        $categories_data = [];
        $sub_categories_data = [];
// return($categories);
        foreach($categories as $x)
        {
            // return(gettype((object)$x));
            $XX = (object)$x;
            array_push($categories_data, $XX->id);
        }
        // return($categories_data);
        $sub_categories = DB::table('categories as c')
                ->select(
                    DB::raw('c.id AS id'),
                    DB::raw('IF(c.name IS NULL, "No Category / بدون تصنيف" ,c.name) as name'),
                    DB::raw('c.parent_id AS parent_id')
                )
                ->whereIn('c.parent_id', $categories_data)
                ->get();
                // return($sub_categories);
        
        $products_data['categories'] = $categories;
        $products_data['sub_categories'] = $sub_categories;
        
        return($products_data);

//         $collection = collect($products_data);
//         $groups = $collection->groupBy('categories_id');
//         $categories_data = $groups->map(function ($row) {

//     return [
//       'categories_id' => $row->first()->categories_id,
//       'category_name' => $row->first()->category_name,
//       'parent_id' => $row->first()->category_name,
//       'total_current_stock' => $row->sum('total_current_stock_qty') ,
//       'total_product_cost' => $row->sum('stock_value_by_cost_price') ,
//       'total_selling_cost' => $row->sum('stock_value_by_selling_price') ,
//       'total_selling_cost_inc_tax' => $row->sum('stock_value_by_selling_price_inc_tax') 
//     ];

//     // return [
//     //   'categories_id' => $row->first()->categories_id,
//     //   'category_name' => $row->first()->category_name,
//     //   'parent_id' => $row->first()->category_name,
//     //   'total_current_stock' => number_format($row->sum('total_current_stock_qty') , 2 ),
//     //   'total_product_cost' => number_format($row->sum('stock_value_by_cost_price') , 2 ),
//     //   'total_selling_cost' => number_format($row->sum('stock_value_by_selling_price') , 2 ),
//     //   'total_selling_cost_inc_tax' => number_format($row->sum('stock_value_by_selling_price_inc_tax') , 2 )
//     // ];
// });
        $totals = [
            'total_current_stock' => $total_current_stock , //total from collections return
            'total_product_cost' => $total_product_cost , //total from collections return
            'total_selling_cost' => $total_selling_cost_without_ince , //total from collections return
            'total_selling_cost_inc_tax' => $total_selling_cost , //total from collections return
        ];

        //return $products;
        //return $categories_data;
        //return $totals;
        $products_data = collect($categories_data);

        $perPage = 10;
        $skip = 0;
        if(!empty($filters['per_page']))
        {
            if (($filters['per_page']) != -1)
            {
                $perPage = $filters['per_page'];
            }
            else
            {
                 $perPage = count($products_data);
            }
        }


        $sells = $this->paginate($categories_data, $perPage);
        $sells->appends(Arr::except($filters, ['page']));

        $response = ((new PaginatedResourceResponse(CollectionResource::make($sells)))->toResponse(request()));
        $data = $response->getData();
        $data->totals =$totals;
        $response->setData($data);
        return $response;

    }

    /**
     * Create a new command instance.
     *
     * @return string
     */
    public function queryToSql($products)
    {
        return vsprintf(str_replace(array('?'), array('\'%s\''), $products->toSql()), $products->getBindings());
    }

    /**
     * The attributes that are mass assignable.
     *
     *
     * @param $items
     * @param int $perPage
     * @param null $page
     * @param array $options
     * @return LengthAwarePaginator
     */
    public function paginate($items, $perPage = 5, $page = null, $options = [])
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(), $perPage, $page, $options);
    }
}
