<?php
namespace Modules\ReportsModule\Http\Controllers;

use stdClass;
use App\User;
use App\Category;
use App\Product;
use Exception;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\PaginatedResourceResponse;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Connector\Http\Controllers\Api\ApiController;
use Modules\ReportsModule\Transformers\CollectionResource;
use Illuminate\Support\Str;
use Carbon\Carbon;

class ProductsMenuReportApiController extends ApiController
{
    /**
     * Display a listing of the resource.
     * @return JsonResponse
     * @throws Exception
     */

    public function index()
    {
        /** @var User $user */
        $user = Auth::user();
        $business_id = $user->business_id;
        $permitted_locations = $user->permitted_locations($business_id);
        $filters = request()->only([
            'location_id'
        ]);
        
        $date = Carbon::now()->setTimezone('Africa/Cairo')->toDateTimeString();
        // return($date);
        
        // Defines
            $products_data = [];
            $categories_id = [];
            $discounts_data = [];
            $obj = new stdClass;
        // End Defines
        
        $products = DB::table('products as p')
                ->select(
                    DB::raw('p.id AS product_id'),
                    DB::raw('p.name AS product_name_ar'),
                    DB::raw('IF(p.product_custom_field1 IS NULL or p.product_custom_field1 = "", p.name , p.product_custom_field1) as product_name_en'),
                    DB::raw('p.product_description AS product_description'),
                    DB::raw('p.image AS product_image'),
                    DB::raw('p.type AS product_type'),
                    DB::raw('p.product_custom_field2 AS product_custom_field2'),
                    DB::raw('p.product_custom_field3 AS product_custom_field3'),
                    DB::raw('p.product_custom_field4 AS product_custom_field4'),
                    DB::raw('p.brand_id AS product_brand_id'),
                    DB::raw('p.category_id AS product_category_id'),
                    DB::raw('p.sub_category_id AS product_sub_category_id')
                )
                ->leftjoin('product_locations as l','l.product_id', '=', 'p.id')
                ->where('p.business_id', $business_id)
                ->where('p.id', "!=" , 1)
                ->where('p.not_for_selling', "!=" , 1)
                ->where('p.is_inactive', "!=" , 1)
                ->orderBy('p.id', 'asc');

                // Filters
                if (!empty($filters['location_id']))
                    $products->whereIn('l.location_id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $products->whereIn('l.location_id', $permitted_locations);
                // End Filters

// return($products->toSql());
                $products = $products->get();
// return($products);
        
        // $discounts = DB::table('discounts as d')
        //         ->select(
        //             DB::raw('d.id AS discount_id'),
        //             DB::raw('d.name AS discount_name'),
        //             DB::raw('d.brand_id AS discount_brand_id'),
        //             DB::raw('d.category_id AS discount_category_id'),
        //             DB::raw('d.discount_type AS discount_type'),
        //             DB::raw('d.priority AS discount_priority'),
        //             DB::raw('d.discount_amount AS discount_amount')
        //         )
        //         // ->leftjoin('discount_variations as dv','dv.discount_id', '=', 'd.id')
        //         ->where('d.business_id', $business_id)
        //         ->where('d.starts_at', '<=' , $date)
        //         ->where('d.ends_at', '>=' , $date)
        //         ->where('d.is_active', 1)
        //         ->orderBy('d.priority', 'desc');
                
        //         // Filters
        //         if (!empty($filters['location_id']))
        //             $discounts->whereIn('d.location_id', explode(',', $filters['location_id']));
        //         else if ($permitted_locations != 'all')
        //             $discounts->whereIn('d.location_id', $permitted_locations);
        //         // End Filters

        // // return([$discounts->toSql() , $discounts->getBindings()]);        
        // $discounts = $discounts->get();
        
        $business_locations = DB::table('business_locations as bl')
                ->select(
                    DB::raw('bl.featured_products AS featured_products')
                )
                ->where('bl.business_id', $business_id);
                
                // Filters
                if (!empty($filters['location_id']))
                    $business_locations->whereIn('bl.id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $business_locations->whereIn('bl.id', $permitted_locations);
                // End Filters

        $business_locations = $business_locations->get();
        

        // foreach($discounts as $discount)
        // {
        //     $discount_variations_data = [];
        //     $discount_variations = DB::table('discount_variations as dv')
        //         ->select(
        //             DB::raw('dv.variation_id AS discount_variation_id')
        //         )
        //         ->where('dv.discount_id', $discount->discount_id)
        //         ->get();
                
        //     foreach($discount_variations as $discount_variation)
        //     {
        //         if(!in_array($discount_variation->discount_variation_id, $discount_variations_data))
        //         {
        //             array_push($discount_variations_data, $discount_variation->discount_variation_id);
        //         }
        //     }

        //     array_push($discounts_data, (object)[
        //             'discount_id' => $discount->discount_id,
        //             'discount_name' => $discount->discount_name,
        //             'discount_brand_id' => $discount->discount_brand_id,
        //             'discount_category_id' => $discount->discount_category_id,
        //             'discount_type' => $discount->discount_type,
        //             'discount_amount' => $discount->discount_amount,
        //             'discount_priority' => $discount->discount_priority,
        //             'discount_variations' => $discount_variations_data,
        //         ]);

        // }
        
        // return($discounts_data);
        
        foreach ($products as $product)
        {
            $is_featured = 0;
            $variations_data = [];
            $variations = DB::table('variations as v')
                ->select(
                    DB::raw('v.id AS variation_id'),
                    DB::raw('v.name AS variation_name'),
                    DB::raw('v.sell_price_inc_tax AS v_dsp_inc_tax')
                    )
                ->where('v.product_id', $product->product_id)
                ->get();
            
            
            foreach($variations as $variation)
            {
                if(in_array($variation->variation_id, (json_decode($business_locations[0]->featured_products)) ))
                $is_featured = 1 ;
                
                array_push($variations_data, (object)[
                    'variation_id' => $variation->variation_id,
                    'variation_name' => $variation->variation_name,
                    'v_dsp_inc_tax' => $variation->v_dsp_inc_tax,
                    'is_featured' => $is_featured,
                ]);
            }
            
                array_push($products_data, (object)[
                    'product_id' => $product->product_id,
                    'product_name_ar' => $product->product_name_ar,
                    'product_name_en' => $product->product_name_en,
                    'product_type' => $product->product_type,
                    'product_description' => $product->product_description,
                    'product_image' => $product->product_image,
                    'product_custom_field2' => $product->product_custom_field2,
                    'product_custom_field3' => $product->product_custom_field3,
                    'product_custom_field4' => $product->product_custom_field4,
                    'product_brand_id' => $product->product_brand_id,
                    'product_category_id' => $product->product_category_id,
                    'product_sub_category_id' => $product->product_sub_category_id,
                    'variations' => $variations_data,
                ]);
                
                if(!in_array($product->product_category_id, $categories_id))
                {
                    array_push($categories_id, $product->product_category_id);
                }

        }
        // return($products_data);
        
        //     $filtered_discount = [];
        // foreach($products_data as $product_data)
        // {
        //     foreach($discounts_data as $discount_data)
        //     {
        //         // return($discount_data->discount_variations_data);
        //         if($product_data->product_brand_id == $discount_data->discount_brand_id && $discount_data->discount_brand_id != null)
        //         {
        //             if(!in_array($discount_data->discount_id, $filtered_discount))
        //                 {
        //                     array_push($filtered_discount, (object)[
        //                         'discount_id' => $discount_data->discount_id,
        //                         'discount_name' => $discount_data->discount_name,
        //                         'discount_brand_id' => $discount_data->discount_brand_id,
        //                         'discount_category_id' => $discount_data->discount_category_id,
        //                         'discount_type' => $discount_data->discount_type,
        //                         'discount_amount' => $discount_data->discount_amount,
        //                         'discount_priority' => $discount_data->discount_priority,
        //                         'discount_variations' => $discount_data->discount_variations,
        //                     ]);
        //                 }
        //         }
        //         elseif($product_data->product_category_id == $discount_data->discount_category_id && $discount_data->discount_category_id != null)
        //         {
        //             if(!in_array($discount_data->discount_id, $filtered_discount))
        //                 {
        //                     array_push($filtered_discount, (object)[
        //                         'discount_id' => $discount_data->discount_id,
        //                         'discount_name' => $discount_data->discount_name,
        //                         'discount_brand_id' => $discount_data->discount_brand_id,
        //                         'discount_category_id' => $discount_data->discount_category_id,
        //                         'discount_type' => $discount_data->discount_type,
        //                         'discount_amount' => $discount_data->discount_amount,
        //                         'discount_priority' => $discount_data->discount_priority,
        //                         'discount_variations' => $discount_data->discount_variations,
        //                     ]);
        //                 }
        //         }
        //         foreach($product_data->variations as $variation)
        //         {
        //             if(in_array($variation->variation_id, $discount_data->discount_variations))
        //             {
        //                 // if(!in_array($discount_data->discount_id, $filtered_discount))
        //                 // {
        //                 //     array_push($filtered_discount, (object)[
        //                 //         'discount_id' => $discount_data->discount_id,
        //                 //         'discount_name' => $discount_data->discount_name,
        //                 //         'discount_brand_id' => $discount_data->discount_brand_id,
        //                 //         'discount_category_id' => $discount_data->discount_category_id,
        //                 //         'discount_type' => $discount_data->discount_type,
        //                 //         'discount_amount' => $discount_data->discount_amount,
        //                 //         'discount_priority' => $discount_data->discount_priority,
        //                 //         'discount_variations' => $discount_data->discount_variations,
        //                 //     ]);
        //                 // }
        //             }
        //         }

        //     }
            
        //     if(count($filtered_discount) > 0)
        //     {
                
                
        //     }
        //     // if($product_data->product_category_id == 1)
        //     // {
        //     // }
        // }

        
        $categories = DB::table('categories as c')
                ->select(
                    DB::raw('c.id AS category_id'),
                    DB::raw('c.name AS category_name'),
                    DB::raw('c.short_code AS category_short_code'),
                    DB::raw('c.parent_id AS category_parent_id'),
                    DB::raw('c.description AS category_description'),
                    DB::raw('c.slug AS category_slug')
                    )
                ->whereIn('c.id', $categories_id)
                ->get();

        
// return([$products_data , $categories]);
        
        $response['data'] = $products_data;
        $response['categories'] = $categories;
        $image_url = "http://erp.adf.house/uploads/img/";
        $response['image_url'] = $image_url;
        
        return $response;
                
            

    }

    /**
     * Create a new command instance.
     *
     * @return string
     */
    public function queryToSql($products)
    {
        return vsprintf(str_replace(array('?'), array('\'%s\''), $products->toSql()), $products->getBindings());
    }

    /**
     * The attributes that are mass assignable.
     *
     *
     * @param $items
     * @param int $perPage
     * @param null $page
     * @param array $options
     * @return LengthAwarePaginator
     */
    public function paginate($items, $perPage = 5, $page = 1, $options = [])
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(), $perPage, $page, $options);
    }
}
