<?php
namespace Modules\ReportsModule\Http\Controllers;

use stdClass;
use App\User;
use App\Category;
use App\Product;
use Exception;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\PaginatedResourceResponse;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Connector\Http\Controllers\Api\ApiController;
use Modules\ReportsModule\Transformers\CollectionResource;
use Illuminate\Support\Str;

class ProductsReportApiController extends ApiController
{
    /**
     * Display a listing of the resource.
     * @return JsonResponse
     * @throws Exception
     */

    public function index()
    {
        /** @var User $user */
        $user = Auth::user();
        $business_id = $user->business_id;
        $permitted_locations = $user->permitted_locations($business_id);
        $filters = request()->only([
            'location_id', 'location_type', 'ratio_filter',
            'not_for_selling', 'is_inactive', 'search', 'managed_stock', 'per_page','page', 'category_id' , 'sub_category_id'
        ]);

        $products = DB::table('products as p')
                ->select(
                    DB::raw('p.id AS product_id'),
                    DB::raw('p.name AS product_name_ar'),
                    DB::raw('IF(p.product_custom_field1 IS NULL or p.product_custom_field1 = "", p.name , p.product_custom_field1) as product_name_en'),
                    DB::raw('v.id AS variation_id'),
                    DB::raw('v.name AS variation_name'),
                    DB::raw('v.sub_sku AS sku'),
                    // DB::raw('c1.name AS category_name'),
                    // DB::raw('c2.name AS sub_category_name'),
                    DB::raw('v.default_purchase_price AS v_dpp'),
                    DB::raw('v.dpp_inc_tax AS v_dpp_inc_tax'),
                    DB::raw('v.default_sell_price as v_dsp'),
                    DB::raw('v.sell_price_inc_tax AS v_dsp_inc_tax'),
                    DB::raw('p.enable_stock AS product_enable_stock')
                )
                ->leftjoin('variations as v','v.product_id', '=', 'p.id')
                // ->leftjoin('categories as c1','p.category_id', '=', 'c1.id')
                // ->leftjoin('categories as c2','p.sub_category_id', '=', 'c2.id')
                ->leftjoin('product_locations as l','l.product_id', '=', 'p.id')
                ->leftjoin('business_locations as bl','bl.id', '=', 'l.location_id')
                ->where('p.business_id', $business_id)
                ->where('p.id', "!=" , 1)
                ->groupBy('v.id')
                ->orderBy('p.id', 'asc');



                // Filters
                if (!empty($filters['location_id']))
                    $products->whereIn('l.location_id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $products->whereIn('l.location_id', $permitted_locations);

                 if (!empty($filters['location_type']))
                 {
                     if($filters['location_type'] == 'null') {
                         $products->whereNull('bl.custom_field1');
                     } else {
                         $products->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                     }
                 }

                if (isset($filters['not_for_selling']) && in_array($filters['not_for_selling'], [0, 1]))
                    $products->where('p.not_for_selling', $filters['not_for_selling']);
                if (isset($filters['is_inactive']) && in_array($filters['is_inactive'], [0, 1]))
                    $products->where('p.is_inactive', $filters['is_inactive']);
                if (isset($filters['managed_stock']) && in_array($filters['managed_stock'], [0, 1]))
                    $products->where('p.enable_stock', $filters['managed_stock']);

                    if (!empty($filters['category_id'])) {
                    $products->where('p.category_id', $filters['category_id']);
                }

                if (!empty($filters['sub_category_id'])) {
                    $products->where('p.sub_category_id', $filters['sub_category_id']);
                }

                if (!empty($filters['search']))
                    $products->where(function ($products) use ($filters) {
                        $products
                            ->where('p.sku', 'like', "%{$filters['search']}%")
                            ->orWhere('p.name', 'like', "%{$filters['search']}%")
                            ->orWhere('p.product_custom_field1', 'like', "%{$filters['search']}%")
                            ->orWhere('v.name', 'like', "%{$filters['search']}%")
                            ->orWhere('v.sub_sku', 'like', "%{$filters['search']}%");
                    });
                // End Filters

                // Query Pagination

                $perPage = 10;
                $skip = 0;
                if(!empty($filters['per_page']))
                {
                    if (($filters['per_page']) != -1) 
                    {
                        $perPage = $filters['per_page'];
                        (isset($filters['page'])) ? $skip = (($filters['page'] -1 ) * $perPage) : $skip = 0;
                        $products->skip($skip)->take($perPage);
                    }
                    else
                    {
                        $perPage = -1;
                        $products->get();
                    }
                }
                else
                {
                    $products->skip($skip)->take($perPage);
                }
        
                // End Query Pagination
// return($products->toSql());
                $products = $products->get();
// return($products);
        // Defines
        $products_data = [];
        $obj = new stdClass;
        $total_current_stock = 0;
        // End Defines
        $total_current_stock = 0;
        $total_product_cost = 0;
        $total_selling_cost = 0;
        $total_unit_profit = 0;
        $total_profit = 0;

        foreach ($products as $product)
        {

            $variation_location_details = DB::table('variation_location_details as vld')
                ->select(
                    DB::raw(' IFNULL( SUM(vld.qty_available) , 0) AS vld_qty_available')
                    )
                ->leftjoin('business_locations as bl','bl.id', '=', 'vld.location_id')
                ->where('vld.variation_id', $product->variation_id)
                ->where('bl.business_id', $business_id);
                if (!empty($filters['location_id']))
                    $variation_location_details->whereIn('vld.location_id', explode(',', $filters['location_id']));


                if (!empty($filters['location_type']))
                {
                    if($filters['location_type'] == 'null') 
                    {
                        $variation_location_details->whereNull('bl.custom_field1');
                    } 
                    else 
                    {
                        $variation_location_details->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                    }
                }


            $variation_location_details = $variation_location_details->get();

            $purchase_lines = DB::table('purchase_lines as pl')
                ->select(
                    DB::raw(' count(pl.id) as pl_count '),
                    DB::raw('  IFNULL( SUM(pl.quantity - pl.quantity_sold - pl.quantity_adjusted - pl.quantity_returned - pl.mfg_quantity_used) , 0) AS pl_qty_available '),
                    DB::raw(' IFNULL( ( SUM( ( pl.quantity - pl.quantity_sold - pl.quantity_adjusted - pl.quantity_returned - pl.mfg_quantity_used ) * pl.purchase_price_inc_tax ) ) , 0) AS pl_qty_available_value_by_pl_avg_product_cost ')
                    )
                ->leftjoin('transactions AS t','t.id', '=', 'pl.transaction_id')
                ->leftjoin('business_locations as bl','bl.id', '=', 't.location_id')
                ->where('t.status', '=' , 'received')
                ->where('pl.variation_id', $product->variation_id);
                
                if (!empty($filters['location_id']))
                    $purchase_lines->whereIn('t.location_id', explode(',', $filters['location_id']));


                if (!empty($filters['location_type']))
                {
                    if($filters['location_type'] == 'null') 
                    {
                        $purchase_lines->whereNull('bl.custom_field1');
                    } 
                    else 
                    {
                        $purchase_lines->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                    }
                }

                $purchase_lines = $purchase_lines->get();


            // VLD Calculations
            $vld_qty_available = $variation_location_details[0]->vld_qty_available ;

            // PL Calculations
            $pl_count = $purchase_lines[0]->pl_count;
            $pl_qty_available = $purchase_lines[0]->pl_qty_available ;

            $pl_qty_available_value_by_pl_avg_product_cost = $purchase_lines[0]->pl_qty_available_value_by_pl_avg_product_cost ;
            $pl_avg_product_cost = ($pl_qty_available_value_by_pl_avg_product_cost>0)?$pl_qty_available_value_by_pl_avg_product_cost / $pl_qty_available : 0;

            // Correction Calculations
            $correction = ($vld_qty_available == $pl_qty_available) ? '0' : '1';
            // End Correction Calculations

            // Product Calculations
            $product_enable_stock = $product->product_enable_stock ;
            $product_dsp = $product->v_dsp ;
            $product_dsp_inc_tax = $product->v_dsp_inc_tax ;
            $product_dpp = $product->v_dpp ;
            $product_dpp_inc_tax = $product->v_dpp_inc_tax ;
            $avg_product_cost = ( $pl_count == 0 || $pl_qty_available  == 0 || $product_enable_stock == 0 || $vld_qty_available < 0 ) ? $product_dpp_inc_tax : $pl_avg_product_cost ;
            // End Product Calculations

            // Stock Values
            $stock_value_by_cost_price = $vld_qty_available * $avg_product_cost;
            $stock_value_by_selling_price = $vld_qty_available * $product_dsp;
            $stock_value_by_selling_price_inc_tax = $vld_qty_available * $product_dsp_inc_tax;
            // End Stock Values

            // Ratio & Profit Calculations
            ($product_dsp_inc_tax == 0) ? $ratio = '0' : $ratio = ( $avg_product_cost / $product_dsp_inc_tax ) ;
            $unit_profit = $product_dsp_inc_tax - $avg_product_cost ;
            $profit = $stock_value_by_selling_price_inc_tax - $stock_value_by_cost_price ;
            // End Ratio & Profit Calculations

            // Total Calculations
            // $total_current_stock += $vld_qty_available;
            // $total_product_cost += $stock_value_by_cost_price;
            // $total_selling_cost += $stock_value_by_selling_price_inc_tax;
            // $total_unit_profit += $unit_profit;
            // $total_profit += $profit;
            // End Total Calculations

                array_push($products_data, (object)[
                    'product_id' => $product->product_id,
                    'product_sku' => $product->sku,
                    'product_name_ar' => $product->product_name_ar,
                    'product_name_en' => $product->product_name_en,
                    'product_variation' => $product->variation_name,

                    'current_stock' => $vld_qty_available,
                    'pl_current_stock' => $pl_qty_available,
                    'correction' => $correction,

                    'avg_product_cost' => $avg_product_cost,
                    'default_selling_price' => $product_dsp,
                    'default_selling_price_inc_tax' => $product_dsp_inc_tax,

                    'stock_value_by_cost_price' => $stock_value_by_cost_price,
                    'stock_value_by_selling_price' => $stock_value_by_selling_price,
                    'stock_value_by_selling_price_inc_tax' => $stock_value_by_selling_price_inc_tax,

                    'ratio' => $ratio * 100,
                    'unit_profit' => $unit_profit,
                    'profit' => $profit,

                    // 'avg_product_cost' => number_format($avg_product_cost , 2 ),
                    // 'default_selling_price' => number_format($product_dsp , 2 ),
                    // 'default_selling_price_inc_tax' => number_format($product_dsp_inc_tax , 2 ),

                    // 'stock_value_by_cost_price' => number_format($stock_value_by_cost_price , 2 ),
                    // 'stock_value_by_selling_price' => number_format($stock_value_by_selling_price , 2 ),
                    // 'stock_value_by_selling_price_inc_tax' => number_format($stock_value_by_selling_price_inc_tax , 2 ),

                    // 'ratio' => number_format($ratio * 100 , 2 ),
                    // 'unit_profit' => number_format($unit_profit , 2 ),
                    // 'profit' => number_format($profit , 2 ),
                ]);


        }

        //  $totals = [
        //     'total_current_stock' => $total_current_stock ,
        //     'total_product_cost' => $total_product_cost ,
        //     'total_selling_cost' => $total_selling_cost ,
        //     'total_unit_profit' => $total_unit_profit ,
        //     'total_profit' => $total_profit ,
        // ];
        
        $products_data = collect($products_data);
        if (isset($filters['ratio_filter']) && $filters['ratio_filter'] == 1)
        {
            $products_data = $products_data->where('ratio' , '>=' , 50);
            
        }
        
        
        // return($products_data);
        // $perPage = 10;
        // $skip = 0;
        // if(!empty($filters['per_page']))
        // {
        //     if (($filters['per_page']) != -1)
        //     {
        //         $perPage = $filters['per_page'];
        //     }
        //     else
        //     {
        //          $perPage = count($products_data);
        //     }
        // }
                //  $perPage = count($products_data);
                
                
        $sells = $this->paginate($products_data, $perPage);
        $sells->appends(Arr::except($filters, ['page']));

        $response = ((new PaginatedResourceResponse(CollectionResource::make($sells)))->toResponse(request()));
        $data = $response->getData();
        // $data->totals =$totals;
        $response->setData($data);
        return $response;
      //return $products_data;
        // return $totals;

    }

    /**
     * Create a new command instance.
     *
     * @return string
     */
    public function queryToSql($products)
    {
        return vsprintf(str_replace(array('?'), array('\'%s\''), $products->toSql()), $products->getBindings());
    }

    /**
     * The attributes that are mass assignable.
     *
     *
     * @param $items
     * @param int $perPage
     * @param null $page
     * @param array $options
     * @return LengthAwarePaginator
     */
    public function paginate($items, $perPage = 5, $page = 1, $options = [])
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(), $perPage, $page, $options);
    }
}
