<?php

namespace Modules\ReportsModule\Http\Controllers;

use stdClass;
use App\User;
use App\Category;
use App\Product;
use Exception;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\PaginatedResourceResponse;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Connector\Http\Controllers\Api\ApiController;
use Modules\ReportsModule\Transformers\CollectionResource;
use Illuminate\Support\Str;

class SalesReportApiController extends ApiController
{
    /**
     * Display a listing of the resource.
     * @return JsonResponse
     * @throws Exception
     */
    public function index()
    {
    // Sending locations according to login user
    $user = Auth::user();
    $business_id = $user->business_id;
    $permitted_locations = $user->permitted_locations($business_id);
    $filters = request()->only([
            'location_id', 'location_type', 'ratio_filter', 'start_date' , 'end_date' ,
            'not_for_selling', 'is_inactive', 'search', 'managed_stock', 'per_page','page', 'category_id' , 'sub_category_id'
        ]);


    // $business_id = request()->session()->get('user.business_id');
    $transactions = DB::table('transactions as t')
                    ->select(
                        DB::raw(' t.id as transaction_id '),
                        // DB::raw(' tsl.id as transaction_sell_line_id '),
                        // DB::raw(' tsl_pl.id as transaction_sell_line_purchase_line_id '),
                        DB::raw(' tsl_pl.purchase_line_id as tsl_pl_purchase_line_id '),
                        DB::raw(' (tsl_pl.quantity - tsl_pl.qty_returned) as tsl_pl_sold_qty '),
                        DB::raw(' tsl.product_id as product_id '),
                        DB::raw(' p.name as product_name_ar '),
                        DB::raw('IF(p.product_custom_field1 IS NULL or p.product_custom_field1 = "", p.name , p.product_custom_field1) as product_name_en'),
                        DB::raw(' tsl.variation_id as variation_id '),
                        DB::raw(' v.name as variation_name '),
                        DB::raw(' v.sub_sku as product_sku '),
                        DB::raw(' (tsl.quantity - tsl.quantity_returned) as tsl_sold_qty '),
                        // DB::raw(' tsl.quantity_returned as tsl_quantity_returned '),
                        DB::raw(' tsl.unit_price as selling_unit_price '),
                        // DB::raw(' tsl.unit_price as tsl_selling_price_without_tax '),
                        DB::raw(' v.dpp_inc_tax as product_purchase_price_inc_tax'),
                        DB::raw(' p.enable_stock AS product_enable_stock')
                        )
                    ->leftjoin('transaction_sell_lines as tsl','t.id', '=', 'tsl.transaction_id')
                    ->leftjoin('transaction_sell_lines_purchase_lines as tsl_pl','tsl.id', '=', 'tsl_pl.sell_line_id')
                    ->leftjoin('products as p','p.id', '=', 'tsl.product_id')
                    ->leftjoin('variations as v','v.id', '=', 'tsl.variation_id')
                    ->leftjoin('business_locations as bl','bl.id', '=', 't.location_id')
                    ->where('t.type', '=' , 'sell')
                    ->where('tsl.variation_id', '!=' , 1)
                    ->where('t.status', '=' , 'final');


                    // Filters
                if (!empty($filters['location_id']))
                    $transactions->whereIn('t.location_id', explode(',', $filters['location_id']));
                else if ($permitted_locations != 'all')
                    $transactions->whereIn('t.location_id', $permitted_locations);

                if (!empty($filters['location_type']))
                {
                    if($filters['location_type'] == 'null') {
                        $transactions->whereNull('bl.custom_field1');
                    } else {
                        $transactions->whereIn('bl.custom_field1', explode(',', $filters['location_type']));
                    }
                }

                if (!empty($filters['start_date']))
                    $transactions->whereDate('t.transaction_date', '>=', $filters['start_date']);
                if (!empty($filters['end_date']))
                    $transactions->whereDate('t.transaction_date', '<=', $filters['end_date']);

                if (!empty($filters['search']))
                $transactions->where(function ($transactions) use ($filters) {
                    $transactions
                        ->where('p.sku', 'like', "%{$filters['search']}%")
                        ->orWhere('p.name', 'like', "%{$filters['search']}%")
                        ->orWhere('p.product_custom_field1', 'like', "%{$filters['search']}%")
                        ->orWhere('v.name', 'like', "%{$filters['search']}%")
                        ->orWhere('v.sub_sku', 'like', "%{$filters['search']}%");
                });

                if (isset($filters['managed_stock']) && in_array($filters['managed_stock'], [0, 1]))
                $transactions->where('p.enable_stock', $filters['managed_stock']);

                if (!empty($filters['category_id'])) {
                    $transactions->where('p.category_id', $filters['category_id']);
                }
        
                if (!empty($filters['sub_category_id'])) {
                    $transactions->where('p.sub_category_id', $filters['sub_category_id']);
                }


    // return($this->queryToSql($transactions));
                $transactions = $transactions->get();
    // return('hello');
    // return($transactions);
    // Defines
    $products_data = [];
    $obj = new stdClass;
    $total_sold_qty = 0;
    $total_product_cost = 0;
    $total_selling_cost = 0;
    // $total_profit = 0;
    // End Defines
        // return($transactions);
    foreach ($transactions as $transaction)
    {
        if($transaction->product_enable_stock == 0)
        {   
            $sold_qty = $transaction->tsl_sold_qty;
            $product_cost = $sold_qty * $transaction->product_purchase_price_inc_tax;
            $selling_cost = $sold_qty * $transaction->selling_unit_price ;
        }
        else
        {
            $purchase_lines = DB::table('purchase_lines as pl')
                    ->select(
                        DB::raw(' pl.purchase_price_inc_tax AS pl_product_cost ')
                        )
                    ->where('pl.id', $transaction->tsl_pl_purchase_line_id)
                    ->get();
            if($transaction->tsl_pl_purchase_line_id != 0 and $purchase_lines->count() > 0 )
            {
                $sold_qty = $transaction->tsl_pl_sold_qty;
                $product_cost = $sold_qty * $purchase_lines[0]->pl_product_cost ;
                $selling_cost = $sold_qty * $transaction->selling_unit_price ;
            }
            else
            {
                if($transaction->tsl_pl_sold_qty == null)
                {
                    $sold_qty = $transaction->tsl_sold_qty;
                }
                else
                {
                    $sold_qty = $transaction->tsl_pl_sold_qty;
                }
                $product_cost =  $sold_qty * $transaction->product_purchase_price_inc_tax;
                $selling_cost = $sold_qty * $transaction->selling_unit_price ;
            }
        }
        

        array_push($products_data, (object)[
            'transaction_id' => $transaction->transaction_id,
            'product_id' => $transaction->product_id,
            'product_sku' => $transaction->product_sku,
            'product_name_ar' => $transaction->product_name_ar,
            'product_name_en' => $transaction->product_name_en,
            'product_variation_id' => $transaction->variation_id,
            'product_variation' => $transaction->variation_name,

            'sold_qty' => $sold_qty,
            'product_cost' => $product_cost,
            // 'product_cost_type' => gettype($product_cost),
            'selling_cost' => $selling_cost,
            // 'selling_cost_type' => gettype($selling_cost),

            // 'avg_product_cost' => number_format($avg_product_cost , 2 ),
            // 'default_selling_price' => number_format($product_dsp , 2 ),
            // 'default_selling_price_inc_tax' => number_format($product_dsp_inc_tax , 2 ),

            // 'stock_value_by_cost_price' => number_format($stock_value_by_cost_price , 2 ),
            // 'stock_value_by_selling_price' => number_format($stock_value_by_selling_price , 2 ),
            // 'stock_value_by_selling_price_inc_tax' => number_format($stock_value_by_selling_price_inc_tax , 2 ),

            // 'ratio' => number_format($ratio * 100 , 2 ),
            // 'unit_profit' => number_format($unit_profit , 2 ),
            // 'profit' => number_format($profit , 2 ),
        ]);

    }
    // return($products_data);
    $collection = collect($products_data);
    $groups = $collection->groupBy('product_variation_id');
    // return($groups);
    $sells_data = $groups->map(function ($row) {
        $sold_qty = $row->sum('sold_qty');
        $total_product_cost = $row->sum('product_cost');
        $avg_product_cost = $total_product_cost / $sold_qty;
        $total_selling_cost = $row->sum('selling_cost');
        if($total_selling_cost != 0)
        {
            $avg_selling_cost = $total_selling_cost / $sold_qty;
            $ratio = ( ( $total_product_cost / $total_selling_cost ) * 100 );
        }
        else
        {
            $avg_selling_cost = 0;
            $ratio = 100;
        }
        $profit = ( $total_selling_cost - $total_product_cost ) ;

        return [
            'product_id' => $row->first()->product_id,
            'product_sku' => $row->first()->product_sku,
            'product_name_ar' => $row->first()->product_name_ar,
            'product_name_en' => $row->first()->product_name_en,
            'product_variation' => $row->first()->product_variation,
            'sold_qty' => $sold_qty ,
            'avg_product_cost' =>  $avg_product_cost ,
            'total_product_cost' => $total_product_cost ,
            'avg_selling_cost' =>  $avg_selling_cost ,
            'total_selling_cost' => $total_selling_cost ,
            'ratio' => $ratio ,
            'profit' => $profit ,
        ];
        
        // return [
        //     'product_id' => $row->first()->product_id,
        //     'product_sku' => $row->first()->product_sku,
        //     'product_name_ar' => $row->first()->product_name_ar,
        //     'product_name_en' => $row->first()->product_name_en,
        //     'product_variation' => $row->first()->product_variation,
        //     'sold_qty' => $row->sum('sold_qty') ,
        //     'avg_product_cost' =>  ($row->sum('product_cost')) / ($row->sum('sold_qty')) ,
        //     'total_product_cost' => $row->sum('product_cost') ,
        //     'avg_selling_cost' =>  ($row->sum('selling_cost')) / ($row->sum('sold_qty')) ,
        //     'total_selling_cost' => $row->sum('selling_cost') ,
        //     // 'ratio' => (( ($row->sum('product_cost')) / ($row->sum('selling_cost')) ) * 100) ,
        //     'profit' => ($row->sum('selling_cost')) - ($row->sum('product_cost')) ,
        //     // 'total_selling_cost_inc_tax' => $row->sum('stock_value_by_selling_price_inc_tax') , 2 )
        // ];
        
        // return [
        //     'product_id' => $row->first()->product_id,
        //     'product_sku' => $row->first()->product_sku,
        //     'product_name_ar' => $row->first()->product_name_ar,
        //     'product_name_en' => $row->first()->product_name_en,
        //     'product_variation' => $row->first()->product_variation,
        //     'sold_qty' => number_format($row->sum('sold_qty') , 2 ),
        //     'avg_product_cost' => number_format( ($row->sum('product_cost')) / ($row->sum('sold_qty')) , 2 ),
        //     'total_product_cost' => number_format($row->sum('product_cost') , 2 ),
        //     'avg_selling_cost' => number_format( ($row->sum('selling_cost')) / ($row->sum('sold_qty')) , 2 ),
        //     'total_selling_cost' => number_format($row->sum('selling_cost') , 2 ),
        //     'ratio' => number_format((($row->sum('product_cost')) / ($row->sum('selling_cost'))) * 100 , 2 ),
        //     'profit' => number_format(($row->sum('selling_cost')) - ($row->sum('product_cost')) , 2 ),
        //     // 'total_selling_cost_inc_tax' => number_format($row->sum('stock_value_by_selling_price_inc_tax') , 2 )
        // ];
    });

    if (isset($filters['ratio_filter']) && $filters['ratio_filter'] == 1)
    {
        $sells_data = $sells_data->where('ratio' , '>=' , 50);
        
    }
    
    
        $perPage = 10;
        $skip = 0;
        if(!empty($filters['per_page']))
        {
            if (($filters['per_page']) != -1)
            {
                $perPage = $filters['per_page'];
            }
            else
            {
                 $perPage = count($sells_data);
            }
        }


        $sells = $this->paginate($sells_data, $perPage);
        $sells->appends(Arr::except($filters, ['page']));

        $response = ((new PaginatedResourceResponse(CollectionResource::make($sells)))->toResponse(request()));
        $data = $response->getData();
        
        // return($data->data);
        
        // foreach($data->data as $a)
        // {
        //     $total_product_cost += number_format($a->total_product_cost , 2);
        // }
        // return($total_product_cost);
        
        // Total Calculations
            $total_sold_qty = array_sum(array_column($data->data, 'sold_qty'));
            $total_product_cost = array_sum(array_column($data->data, 'total_product_cost'));
            $total_selling_cost = array_sum(array_column($data->data, 'total_selling_cost'));
        // End Total Calculations

        // return($products_data->sum('product_cost'));

        $totals = [
            'total_sold_qty' => $total_sold_qty , //total from collections return
            'total_product_cost' => $total_product_cost , //total from collections return
            'total_selling_cost' => $total_selling_cost , //total from collections return
            // 'total_selling_cost_inc_tax' => number_format($total_selling_cost,2), //total from collections return
            'total_profit' => $total_selling_cost - $total_product_cost , //total from collections return
        ];

        $data->totals =$totals;

        // return $data->data;
        
        
        $response->setData($data);
        return $response;


    }


    /**
     * Create a new command instance.
     *
     * @return string
     */
    public function queryToSql($query)
    {
        return vsprintf(str_replace(array('?'), array('\'%s\''), $query->toSql()), $query->getBindings());
    }

    /**
     * The attributes that are mass assignable.
     *
     *
     * @param $items
     * @param int $perPage
     * @param null $page
     * @param array $options
     * @return LengthAwarePaginator
     */
    public function paginate($items, $perPage = 5, $page = null, $options = [])
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(), $perPage, $page, $options);
    }
}
